import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsPBPUsage(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.pbpusage'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "file://${P}.patch"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://${P}.com/${P}.git"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "https://foo.org/${P}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORDKIR}/${P}"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pbpusage'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "file://${P}.patch"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://${P}.com/${P}.git"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "https://foo.org/${P}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORDKIR}/${P}"',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.pbpusage'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "file://${BP}.patch"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://${BP}.com/${BP}.git"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "https://foo.org/${BP}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORDKIR}/${BP}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRCREV_${P} = "01234567890abcdef"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsSRCURIappend(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.srcuriappend'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "file://abc"
                                     inherit abc
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI  += "file://abc"
                                     inherit abc
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcuriappend'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI  += "file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI_append = " file://abc"
                                     inherit abc
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI_remove = "file://abc"
                                     inherit abc
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:append = " file://abc"
                                     inherit abc
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:remove = "file://abc"
                                     inherit abc
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI_prepend = "file://abc "
                                     inherit abc
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit abc
                                     SRC_URI[md5sum] = "1234"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit abc
                                     SRC_URI[sha256sum] = "1234"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsSRCURIReqOpt(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "git://foo.org/gaz.git;name=foo;branch=master"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "gitsm://foo.org/gaz.git;name=foo;branch=master"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "git://foo.org/gaz.git;name=foo;protocol=ssh"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "gitsm://foo.org/gaz.git;name=foo;protocol=ssh"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [2])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "git://foo.org/gaz.git;name=foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "gitsm://foo.org/gaz.git;name=foo"',
                                 },
                             ],
                             )
    def test_really_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "git://foo.org/gaz.git;name=foo;protocol=ssh;branch=main"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "gitsm://foo.org/gaz.git;name=foo;protocol=ssh;branch=main"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "git://foo.org/gaz.git;name=foo;protocol=ssh;nobranch=1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "gitsm://foo.org/gaz.git;name=foo;protocol=ssh;nobranch=1"',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     SRC_REPO ?= "bitbucket.org/foo/bar.git"
                                     GIT_PROTO ?= "user=git;protocol=ssh"
                                     GIT_SUBPATH ?= "subpath=tools/${PACKAGE_NAME}"
                                     SRC_URI = "git://${SRC_REPO};nobranch=1;${GIT_PROTO};${GIT_SUBPATH}"
                                     SRCREV ?= "master"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.parser import INLINE_BLOCK

from oelint_adv.cls_rule import Rule


class VarSRCUriOptions(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.srcuridomains',
                         severity='warning',
                         message='Recipe is pulling from different domains, this will likely cause issues')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR, attributeValue='SRC_URI')

        _domains = {'default': set()}
        _override_ignores = ['append', 'remove', 'prepend']
        for item in items:
            _overrides = '-'.join(sorted([x for x in item.SubItems if x not in _override_ignores])) or 'default'
            _is_append = item.IsAppend()
            _is_remove = 'remove' in item.AppendOperation()
            if _overrides not in _domains:
                _domains[_overrides] = set()
            for u in [x.strip('\'').strip() for x in item.get_items()]:
                if u == INLINE_BLOCK:
                    continue
                _url = stash.GetScrComponents(u)
                if _url['scheme'] and _url['scheme'] not in ['file']:
                    domain = _url['src'].split('/')[0]
                    if _is_append:
                        _domains[_overrides].add(domain)
                    elif _is_remove and domain in _domains[_overrides]:
                        _domains[_overrides].remove(domain)
                    else:
                        _domains[_overrides] = {domain}
        for _, v in _domains.items():
            if len(v) > 1:
                res += self.finding(item.Origin, item.InFileLine)
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Inherit, Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.constants import CONSTANTS

from oelint_adv.cls_rule import Rule


class VarMandatoryExists(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.mandatoryvar',
                         severity='error',
                         message='<FOO>',
                         onappend=False,
                         appendix=CONSTANTS.VariablesMandatory)
        CONSTANTS.AddConstants(
            {
                'oelint-mandatoryvar':
                {
                    'image-excludes': [
                        'CVE_PRODUCT',
                        'HOMEPAGE',
                        'SRC_URI',
                    ],
                },
            },
        )

        CONSTANTS.AddConstants(
            {
                'oelint-mandatoryvar':
                {
                    'pkggroup-excludes': [
                        'CVE_PRODUCT',
                        'HOMEPAGE',
                        'LICENSE',
                        'SRC_URI',
                    ],
                },
            },
        )

        CONSTANTS.AddConstants(
            {
                'oelint-mandatoryvar':
                {
                    'SRC_URI-exclude-classes': [
                        'pypi',
                        'gnomebase',
                    ],
                },
            },
        )

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        _is_pkg_group = stash.IsPackageGroup(_file)
        _is_image = stash.IsImage(_file)
        items: List[Variable] = stash.GetItemsFor(filename=_file,
                                                  classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR,
                                                  attributeValue=CONSTANTS.VariablesMandatory)
        inherits: List[Inherit] = stash.GetItemsFor(filename=_file,
                                                    classifier=Inherit.CLASSIFIER)

        def get_class_specific(varname: str, inherits: List[Inherit]) -> bool:
            return any(True for x in inherits if any(y in x.get_items()
                                                     for y in (CONSTANTS.GetByPath(f'oelint-mandatoryvar/{varname}-exclude-classes') or [])))

        # some classes set SRC_URI on their own, do not warn here
        for var_ in CONSTANTS.VariablesMandatory:
            if _is_pkg_group and var_ in CONSTANTS.GetByPath('oelint-mandatoryvar/pkggroup-excludes'):
                continue
            if _is_image and var_ in CONSTANTS.GetByPath('oelint-mandatoryvar/image-excludes'):
                continue
            if get_class_specific(var_, inherits):
                continue
            filtered = stash.Reduce(items, attribute=Variable.ATTR_VAR, attributeValue=var_)
            if not any(filtered):
                res += self.finding(_file, 0,
                                    'Variable \'{a}\' should be set'.format(a=var_), appendix=var_)
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarMandatoryVar(TestBaseClass):

    def __generate_sample_code(self, var, extra):
        return '''
            {extra}
            {var} = "foo"
            '''.format(var=var, extra=extra)

    @pytest.mark.parametrize('id_', ['oelint.var.mandatoryvar'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        'SUMMARY',
        'DESCRIPTION',
        'HOMEPAGE',
        'LICENSE',
        'SRC_URI',
    ])
    def test_bad(self, id_, occurrence, var):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code('A', ''),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.mandatoryvar'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        'SUMMARY',
        'DESCRIPTION',
        'LICENSE',
    ])
    @pytest.mark.parametrize('extra', [
        'IMAGE_INSTALL_append = " foo"',
        'inherit image',
        'inherit core-image',
        'IMAGE_INSTALL += " foo"',
        'IMAGE_INSTALL = "foo"',
    ])
    def test_bad_image(self, id_, occurrence, var, extra):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code('A', extra),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', [
        'oelint.var.mandatoryvar.SUMMARY',
        'oelint.var.mandatoryvar.DESCRIPTION',
        'oelint.var.mandatoryvar.HOMEPAGE',
        'oelint.var.mandatoryvar.LICENSE',
        'oelint.var.mandatoryvar.SRC_URI',
    ])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SUMMARY = "foo"
                                     DESCRIPTION = "foo"
                                     HOMEPAGE = "foo"
                                     LICENSE = "foo"
                                     SRC_URI = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit core-image
                                     SUMMARY = "foo"
                                     DESCRIPTION = "foo"
                                     LICENSE = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit image
                                     SUMMARY = "foo"
                                     DESCRIPTION = "foo"
                                     LICENSE = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit packagegroup
                                     SUMMARY = "foo"
                                     DESCRIPTION = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SUMMARY = "foo"
                                     DESCRIPTION = "foo"
                                     HOMEPAGE = "foo"
                                     LICENSE = "foo"
                                     inherit pypi
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SUMMARY = "foo"
                                     DESCRIPTION = "foo"
                                     HOMEPAGE = "foo"
                                     LICENSE = "foo"
                                     inherit gnomebase
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', [
        'oelint.var.mandatoryvar.SRC_URI',
    ])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SUMMARY = "foo"
                                     DESCRIPTION = "foo"
                                     LICENSE = "foo"
                                     inherit foo
                                     ''',
                                 },
                             ],
                             )
    def test_good_custom_varname_class(self, input_, id_, occurrence):
        __cnt = '''
        {
            "oelint-mandatoryvar": {
                "SRC_URI-exclude-classes": [
                    "foo"
                ],
                "HOMEPAGE-exclude-classes": [
                    "foo"
                ]
            }
        }
        '''
        _extra_opts = [
            '--constantmod=+{mod}'.format(mod=self._create_tempfile('constmod', __cnt))]
        self.check_for_id(self._create_args(input_, _extra_opts), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsUnpackdir(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.unpackdir'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORKDIR}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     B = "${WORKDIR}/foo/bar"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_new(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=styhead']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.unpackdir'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORKDIR}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     B = "${WORKDIR}/foo/bar"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.unpackdir'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     S = "${UNPACKDIR}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     B = "${UNPACKDIR}/foo/bar"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SOMEOTHER = "${WORKDIR}/foo/bar"
                                     ''',
                                 },
                             ],
                             )
    def test_good_inherit(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=styhead']), id_, occurrence)



import os

import pytest  # noqa: I900

from .base import TestBaseClass
from oelint_adv.core import deserialize_boolean_options

# flake8: noqa S101 - n.a. for test files


class TestConfigFile(TestBaseClass):

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_environ(self, input_):
        # Test the default
        _args = self._create_args(input_)

        assert not _args.hide

        # test the override from config file
        # here loaded via environment variable
        _cstfile = self._create_tempfile('oelint.cfg', '[oelint]\nhide=warning')
        os.environ['OELINT_CONFIG'] = _cstfile
        _args = self._create_args(input_)
        del os.environ['OELINT_CONFIG']

        assert _args.hide == ['warning']

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_home(self, input_):
        # Test the default
        _args = self._create_args(input_)

        assert not _args.hide

        # test the override from config file
        # here loaded via home folder
        _cstfile = self._create_tempfile(
            '.oelint.cfg', '[oelint]\nhide=warning')
        os.environ['HOME'] = os.path.dirname(_cstfile)
        _args = self._create_args(input_)

        assert _args.hide == ['warning']

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_workdir(self, input_):
        # Test the default
        _args = self._create_args(input_)

        assert not _args.hide

        # test the override from config file
        # here loaded from current workdir
        _cstfile = self._create_tempfile(
            '.oelint.cfg', '[oelint]\nhide=warning')

        _cwd = os.getcwd()
        os.chdir(os.path.dirname(_cstfile))
        _args = self._create_args(input_)
        os.chdir(_cwd)

        assert _args.hide == ['warning']

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_skip(self, input_):
        # Test the default
        _args = self._create_args(input_)

        assert not _args.hide

        # test the override from config file
        # here loaded from current workdir
        _cstfile = self._create_tempfile(
            '.oelint.cfg', '[oelint]\nhide=warning')
        
        os.environ['OELINT_SKIP_CONFIG'] = '1'

        _cwd = os.getcwd()
        os.chdir(os.path.dirname(_cstfile))
        _args = self._create_args(input_)
        os.chdir(_cwd)

        os.environ.pop('OELINT_SKIP_CONFIG')

        assert not _args.hide

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_boolean_options(self, input_):
        # Test if the following options are auto converted
        # to boolean arguments
        for _option in [
            'color',
            'exit-zero',
            'fix',
            'nobackup',
            'noinfo',
            'nowarn',
            'print-rulefile',
            'quiet',
            'relpaths',
        ]:
            _cstfile = self._create_tempfile(
                '.oelint.cfg', '[oelint]\n{item}=True'.format(item=_option))
            os.environ['OELINT_CONFIG'] = _cstfile
            _args = self._create_args(input_)
            del os.environ['OELINT_CONFIG']

            assert isinstance(getattr(_args, _option.replace('-', '_')), bool)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_no_convert(self, input_):
        # Test if the following options remain untouched
        for _option in [
            'output',
            'messageformat',
        ]:
            _cstfile = self._create_tempfile(
                '.oelint.cfg', '[oelint]\n{item}=True'.format(item=_option))
            os.environ['OELINT_CONFIG'] = _cstfile
            _args = self._create_args(input_)
            del os.environ['OELINT_CONFIG']

            assert getattr(_args, _option) == True

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_multiple(self, input_):
        # Test if the following options are converted to lists
        for _option in [
            'addrules',
            'customrules',
            'suppress',
            'hide',
        ]:
            _cstfile = self._create_tempfile(
                '.oelint.cfg', '[oelint]\n{item}=\t+True\n\t-False'.format(item=_option))
            os.environ['OELINT_CONFIG'] = _cstfile
            _args = self._create_args(input_)

            assert isinstance(getattr(_args, _option), list)
            assert getattr(_args, _option) == ['+True', '-False']

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_messageformat(self, input_):
        _cstfile = self._create_tempfile(
            '.oelint.cfg', '[oelint]\nmessageformat={severity}-{id}-{msg}')
        os.environ['OELINT_CONFIG'] = _cstfile
        _args = self._create_args(input_)
        del os.environ['OELINT_CONFIG']

        assert getattr(_args, 'messageformat') == '{severity}-{id}-{msg}'

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_environ_broken(self, input_):
        # Test the default
        _args = self._create_args(input_)

        assert not _args.hide

        # test the override from config file
        # here loaded via environment variable but with a broken file
        _cstfile = self._create_tempfile('oelint.cfg', '[oel]\nhide=')
        os.environ['OELINT_CONFIG'] = _cstfile
        _args = self._create_args(input_)
        del os.environ['OELINT_CONFIG']

        assert not _args.hide

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb': 'VAR = "1"',
                                 },
                             ],
                             )
    def test_config_file_dash_replace(self, input_):
        # Test if the following options do the needed
        # - -> _ replacements automatically
        for _option in [
            'exit-zero',
            'print-rulefile',
        ]:
            _args = self._create_args(input_)
            assert not getattr(_args, _option.replace('-', '_'))

            _cstfile = self._create_tempfile(
                '.oelint.cfg', '[oelint]\n{item}=True'.format(item=_option))
            os.environ['OELINT_CONFIG'] = _cstfile
            _args = self._create_args(input_)
            del os.environ['OELINT_CONFIG']

            assert getattr(_args, _option.replace('-', '_'))


    def test_option_deserialization(self):
        options = {'a': 'True', 'b': True, 'c': 'False', 'd': False, 'e': 'other value'}
        deserialized = deserialize_boolean_options(options)
        assert deserialized == {'a': True, 'b': True, 'c': False, 'd': False, 'e': 'other value'}



from typing import List, Tuple

from oelint_parser.cls_item import Inherit
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarInheritDevtool(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.inheritdevtool',
                         severity='warning',
                         message='It is better to use inherit_defer for {statement}, to enable proper devtool integration',
                         appendix=['native', 'nativesdk', 'cross'],
                         valid_from_release='scarthgap')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Inherit] = stash.GetItemsFor(filename=_file, classifier=Inherit.CLASSIFIER)
        for i in items:
            if 'native' in i.get_items() and i.Statement != 'inherit_defer':
                res += self.finding(i.Origin, i.InFileLine,
                                    override_msg=self.Msg.format(statement='native'),
                                    appendix='native',
                                    )
            if 'nativesdk' in i.get_items() and i.Statement != 'inherit_defer':
                res += self.finding(i.Origin, i.InFileLine,
                                    override_msg=self.Msg.format(statement='nativesdk'),
                                    appendix='nativesdk',
                                    )
            if 'cross' in i.get_items() and i.Statement != 'inherit_defer':
                res += self.finding(i.Origin, i.InFileLine,
                                    override_msg=self.Msg.format(statement='cross'),
                                    appendix='cross',
                                    )
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Inherit, Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarBbclassextend(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.bbclassextend',
                         severity='info',
                         message='BBCLASSEXTEND should be set if possible')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        if stash.IsPackageGroup(_file) or stash.IsImage(_file):
            return []
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR, attributeValue='BBCLASSEXTEND')
        items_inherit: List[Inherit] = stash.GetItemsFor(filename=_file, classifier=Inherit.CLASSIFIER)
        if not any(items):
            _safe = False
            for item in items_inherit:
                if any(x in ['native', 'nativesdk', 'cross'] for x in item.get_items()):
                    _safe = True
                    break
            if not _file.endswith('.bbappend') and not _safe:
                res += self.finding(_file, 0)
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsSRCURIRevTag(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurisrcrevtag'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://foo.org/gaz.git;tag=${PV};name=foo"
                                     SRCREV_foo = "1234"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://foo.org/gaz.git;tag=${PV}"
                                     SRCREV = "1234"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurisrcrevtag'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://foo.org/gaz.git;name=bar"
                                     SRCREV_bar = "1234"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://foo.org/gaz.git"
                                     SRCREV = "1234"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://foo.org/gaz.git;tag=${PV};name=foo"
                                     SRCREV_bar = "1234"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI[md5sum] = "file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI[sha256sum] = "file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "${@magic.foo.operation}"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass

# flake8: noqa W291 - we want to explicitly test trailing whitespace here
class TestClassOelintSpacesLineEnd(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.spaces.lineend'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     ABC = "1" 
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     # just a comment 
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.spaces.lineend'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     ABC = "1" 
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     # just a comment 
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     ABC = "1" 
                                     ABC += "2" 
                                     ''',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.spaces.lineend'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     ABC = "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEF = "${@foo_magic_grill(d)}"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List
from colorama import Fore
from colorama.ansi import AnsiCodes


class State():
    """State/Configuration shared between processes."""

    def __init__(self) -> None:
        self.color = False
        self.inline_suppressions = {}
        self.messageformat = ''
        self.hide = {'error': False, 'warning': False, 'info': False}
        self.rel_path = False
        self.rule_file = {}
        self.suppression = []
        self.nobackup = False
        self.additional_stash_args = {}

        self.__colors_by_severity = {
            'info': Fore.GREEN,
            'warning': Fore.YELLOW,
            'error': Fore.RED,
        }

        self._seen_inline_suppressions = []

    def get_colorize(self) -> bool:
        """Returns weather or not the terminal output is to be colorized"""
        return self.color

    def get_color_by_severity(self, severity: str) -> AnsiCodes:
        """Get an ANSI color code given a severity

        Args:
            severity (str): either `neutral`, `info`, `warning` or `error`
        """
        return self.__colors_by_severity.get(severity, '')

    def get_hide(self, severity) -> bool:
        """--hide severity is set

        Returns:
            bool: hide messages of given severity
        """
        return self.hide[severity]

    def get_relpaths(self) -> bool:
        """--relpath flag is set

        Returns:
            bool: relpath is set
        """
        return self.rel_path

    def get_messageformat(self) -> str:
        """Get messageformat

        Returns:
            str: message format
        """
        return self.messageformat

    def get_suppressions(self) -> List[str]:
        """Get suppressions

        Returns:
            List[str]: Set suppressions
        """
        return self.suppression

    def get_rulefile(self) -> dict:
        """Rule file

        Returns:
            dict: set rule file
        """
        return self.rule_file

    def get_inlinesuppressions(self) -> dict:
        """Inline suppressions

        Returns:
            dict: Set inline suppressions
        """
        return self.inline_suppressions

    def get_nobackup(self) -> bool:
        """nobackup set

        Returns:
            bool: nobackup is set
        """
        return self.nobackup  # pragma: no cover

    def get_additional_stash_args(self) -> dict:
        """Additional arguments for the Stash init

        Returns:
            dict: additional args
        """
        return self.additional_stash_args

    def set_inline_suppression_seen(self, file: str, line: int, ids: List[str]) -> None:
        """Announce that inline suppression have been used

        Args:
            file (str): File path
            line (int): Line
            ids (List[str]): IDs of suppression
        """
        self._seen_inline_suppressions += [(file, line, x) for x in ids]

    def get_inline_suppression_seen(self, file: str, line: int, id_: str) -> bool:
        """Check if inline suppression have been used

        Args:
            file (str): File path
            line (int): Line
            id_ (str): ID of suppression

        Returns:
            bool: True if found
        """
        return (file, line, id_) in self._seen_inline_suppressions



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsSpecific(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.specific'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A:append:fooarch = " abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'B:bararch += "abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     A:append:fooarch = " abc"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     B:bararch += "abc"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.specific'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A_append_fooarch = " abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'B_bararch += "abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     A_append_fooarch = " abc"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     B_bararch += "abc"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.specific'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "foo"
                                     A:append:fooarch = " abc"
                                     COMPATIBLE_MACHINE += "|bar"
                                     B:bararch += "abc"
                                     PACKAGES =+ "${PN}-httpd ${PN}-syslog ${PN}-mdev ${PN}-udhcpd ${PN}-udhcpc ${PN}-hwclock"
                                     FILES:${PN}-httpd = "${sysconfdir}/init.d/busybox-httpd /srv/www"
                                     FILES:${PN}-syslog = "${sysconfdir}/init.d/syslog* ${sysconfdir}/syslog-startup.conf* ${sysconfdir}/syslog.conf* ${systemd_unitdir}/system/syslog.service ${sysconfdir}/default/busybox-syslog"
                                     FILES:${PN}-mdev = "${sysconfdir}/init.d/mdev ${sysconfdir}/mdev.conf ${sysconfdir}/mdev/*"
                                     FILES:${PN}-udhcpd = "${sysconfdir}/init.d/busybox-udhcpd"
                                     FILES:${PN}-udhcpc = "${sysconfdir}/udhcpc.d ${datadir}/udhcpc"
                                     FILES:${PN}-hwclock = "${sysconfdir}/init.d/hwclock.sh"
                                     DO_IPv4 := "${@bb.utils.contains('DISTRO_FEATURES', 'ipv4', 1, 0, d)}"
                                     DO_IPv6 := "${@bb.utils.contains('DISTRO_FEATURES', 'ipv6', 1, 0, d)}"
                                     SRC_URI:append:libc-musl = " file://musl.cfg "
                                     SRC_URI[tarball.md5sum] = "0a367e19cdfd157e8258d87f893ee516"
                                     SRC_URI[tarball.sha256sum] = "97648636e579462296478e0218e65e4bc1e9cd69089a3b1aeb810bff7621efb7"
                                     PACKAGES += "${PN}-foo myrecipe-foo"
                                     USERADD_PARAM:myrecipe-foo = "--bar --baz"
                                     SRC_URI = "\\
                                         git://foo.org/baz.git;name=super \\
                                         git://foo.org/bar.git;name=ultra \\
                                     "
                                     SRC_URI += "git://foo.org/${BPN}.git;name=${PN}"
                                     SRCREV_super = "2a76ac0ff0e702a7f553b6d7135a1089e9c3b469"
                                     SRCREV_ultra = "7533c21ba6c06a513eaa9500a06ae780249b9834"
                                     PACKAGECONFIG_class-nativesdk ??= "${PACKAGECONFIG_class-native}"
                                     PACKAGES =+ "libpulsecore libpulsecommon libpulse libpulse-simple libpulse-mainloop-glib \\
                                                 pulseaudio-server pulseaudio-misc ${@bb.utils.contains('PACKAGECONFIG', 'dbus', 'pulseaudio-module-console-kit', '', d)}"
                                     DEPENDS:pulseaudio-server = "foo-bar"
                                     DEPENDS:clang = "Unknown"
                                     DEPENDS:linux = "Linux"
                                     DEPENDS:darwin = "Darwin"
                                     DEPENDS:mingw32 = "Windows"
                                     A:oelint = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGE_BEFORE_PN =. "extra-pkg"
                                     FILES:extra-pkg = "${base_bindir}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGE_BEFORE_PN =. "extra-pkg"
                                     FILES:extra-pkg = "${base_bindir}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     A:append:fooarch = " abc"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     B:bararch += "abc"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'B:poky += "abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'B:nodistro += "abc"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.specific'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "foo"
                                     A_append_fooarch = " abc"
                                     COMPATIBLE_MACHINE += "|bar"
                                     B_bararch += "abc"
                                     PACKAGES =+ "${PN}-httpd ${PN}-syslog ${PN}-mdev ${PN}-udhcpd ${PN}-udhcpc ${PN}-hwclock"
                                     FILES_${PN}-httpd = "${sysconfdir}/init.d/busybox-httpd /srv/www"
                                     FILES_${PN}-syslog = "${sysconfdir}/init.d/syslog* ${sysconfdir}/syslog-startup.conf* ${sysconfdir}/syslog.conf* ${systemd_unitdir}/system/syslog.service ${sysconfdir}/default/busybox-syslog"
                                     FILES_${PN}-mdev = "${sysconfdir}/init.d/mdev ${sysconfdir}/mdev.conf ${sysconfdir}/mdev/*"
                                     FILES_${PN}-udhcpd = "${sysconfdir}/init.d/busybox-udhcpd"
                                     FILES_${PN}-udhcpc = "${sysconfdir}/udhcpc.d ${datadir}/udhcpc"
                                     FILES_${PN}-hwclock = "${sysconfdir}/init.d/hwclock.sh"
                                     DO_IPv4 := "${@bb.utils.contains('DISTRO_FEATURES', 'ipv4', 1, 0, d)}"
                                     DO_IPv6 := "${@bb.utils.contains('DISTRO_FEATURES', 'ipv6', 1, 0, d)}"
                                     SRC_URI_append_libc-musl = " file://musl.cfg "
                                     SRC_URI[tarball.md5sum] = "0a367e19cdfd157e8258d87f893ee516"
                                     SRC_URI[tarball.sha256sum] = "97648636e579462296478e0218e65e4bc1e9cd69089a3b1aeb810bff7621efb7"
                                     PACKAGES += "${PN}-foo myrecipe-foo"
                                     USERADD_PARAM_myrecipe-foo = "--bar --baz"
                                     SRC_URI = "\\
                                         git://foo.org/baz.git;name=super \\
                                         git://foo.org/bar.git;name=ultra \\
                                     "
                                     SRC_URI += "git://foo.org/${BPN}.git;name=${PN}"
                                     SRCREV_super = "2a76ac0ff0e702a7f553b6d7135a1089e9c3b469"
                                     SRCREV_ultra = "7533c21ba6c06a513eaa9500a06ae780249b9834"
                                     PACKAGECONFIG_class-nativesdk ??= "${PACKAGECONFIG_class-native}"
                                     PACKAGES =+ "libpulsecore libpulsecommon libpulse libpulse-simple libpulse-mainloop-glib \\
                                                 pulseaudio-server pulseaudio-misc ${@bb.utils.contains('PACKAGECONFIG', 'dbus', 'pulseaudio-module-console-kit', '', d)}"
                                     DEPENDS_pulseaudio-server = "foo-bar"
                                     DEPENDS_clang = "Unknown"
                                     DEPENDS_linux = "Linux"
                                     DEPENDS_darwin = "Darwin"
                                     DEPENDS_mingw32 = "Windows"
                                     A_oelint = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGE_BEFORE_PN =. "extra-pkg"
                                     FILES_extra-pkg = "${base_bindir}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGE_BEFORE_PN =. "extra-pkg"
                                     FILES_extra-pkg = "${base_bindir}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     A_append_fooarch = " abc"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     B_bararch += "abc"
                                     ''',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsSectionLowerCase(TestBaseClass):

    def __generate_sample_code(self, op):
        return '''
            VAR{op}"1"
            '''.format(op=op)

    @pytest.mark.parametrize('id_', ['oelint.vars.spacesassignment'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('op',
                             [
                                 ' :=',
                                 ' ??=',
                                 ' ?=',
                                 ' .=',
                                 ' +=',
                                 ' =.',
                                 ' =',
                                 ' =+',
                                 ':= ',
                                 ':=',
                                 '??= ',
                                 '??=',
                                 '?= ',
                                 '?=',
                                 '.= ',
                                 '.=',
                                 '+= ',
                                 '+=',
                                 '= ',
                                 '=. ',
                                 '=.',
                                 '=',
                                 '=+ ',
                                 '=+',
                             ],
                             )
    def test_bad(self, op, id_, occurrence):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(op),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.spacesassignment'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('op',
                             [
                                 ' = ',
                                 ' =. ',
                                 ' .= ',
                                 ' += ',
                                 ' =+ ',
                                 ' ?= ',
                                 ' ??= ',
                                 ' := ',
                             ],
                             )
    def test_good(self, op, id_, occurrence):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(op),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import FunctionExports
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class ExportFunctionsDash(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.exportfunction.dash',
                         severity='error',
                         message='EXPORT_FUNCTIONS should not contain \'-\', replace by \'_\'.')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        for item in stash.GetItemsFor(filename=_file,
                                      classifier=FunctionExports.CLASSIFIER):
            if any('-' in x for x in item.get_items()):
                res += self.finding(_file, item.InFileLine)
        return res



import argparse
import json
import os
from argparse import ArgumentTypeError

import pytest  # noqa: I900

from .base import TestBaseClass


# flake8: noqa S101 - n.a. for test files
class TestClassIntegration(TestBaseClass):

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_color_input(self, input_):
        # local imports only
        from colorama import Fore
        from oelint_adv.__main__ import run

        _args = self._create_args(input_, extraopts=['--color'])
        issues = [x[1] for x in run(_args)]
        issues_formatted = '\n'.join(issues)
        assert (any(Fore.RED in x for x in issues)
                ), f'red expected in:\n{issues_formatted}'
        assert (any(Fore.YELLOW in x for x in issues)
                ), f'yellow expected in:\n{issues_formatted}'
        assert (any(Fore.GREEN in x for x in issues)
                ), f'green expected in:\n{issues_formatted}'

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_relpaths(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(input_, extraopts=['--relpaths'])
        _cwd = os.getcwd()
        os.chdir('/tmp')  # noqa: S108 - usage of tmp is fine for our purposes
        issues = [x[1] for x in run(_args)]
        assert (not any(x.startswith('/tmp/') for x in issues))  # noqa: S108 - usage of tmp is fine for our purposes
        os.chdir(_cwd)

    def test_missing_file_args(self):
        with pytest.raises(ArgumentTypeError):
            self._create_args({})

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'VAR = "1"',
                                 },
                             ],
                             )
    def test_constmod_add(self, input_):
        # local imports only
        from oelint_parser.constants import CONSTANTS

        __cnt = '''
        {
            "functions": {
                "known": [
                    "do_foo_bar"
                ]
            }
        }
        '''
        _extra_opts = [
            '--constantmod=+{mod}'.format(mod=self._create_tempfile('constmod', __cnt))]
        _args = self._create_args(input_, extraopts=_extra_opts)

        assert ('do_foo_bar' in CONSTANTS.FunctionsKnown)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'VAR = "1"',
                                 },
                             ],
                             )
    def test_constmod_remove(self, input_):
        # local imports only
        from oelint_parser.constants import CONSTANTS

        __cnt = '''
        {
            "functions": {
                "known": [
                    "do_ar_patched"
                ]
            }
        }
        '''
        _extra_opts = [
            '--constantmod=-{mod}'.format(mod=self._create_tempfile('constmod', __cnt))]
        self._create_args(input_, extraopts=_extra_opts)

        assert ('do_ar_patched' not in CONSTANTS.FunctionsKnown)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'VAR = "1"',
                                 },
                             ],
                             )
    def test_constmod_override(self, input_):
        # local imports only
        from oelint_parser.constants import CONSTANTS

        __cnt = '''
        {
            "functions": {
                "known": [
                    "do_ar_patched"
                ]
            }
        }
        '''
        _extra_opts = [
            '--constantmod={mod}'.format(mod=self._create_tempfile('constmod', __cnt))]
        _args = self._create_args(input_, extraopts=_extra_opts)

        assert (['do_ar_patched'] == CONSTANTS.FunctionsKnown)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'VAR = "1"',
                                 },
                             ],
                             )
    def test_constmod_corrupt(self, input_):
        __cnt = '''
        {
            "functions": 
                "known": [
                    "do_ar_patched"
                ]
            }
        }
        '''
        _extra_opts = [
            '--constantmod={}'.format(self._create_tempfile('constmod', __cnt))]
        with pytest.raises(ArgumentTypeError):
            self._create_args(input_, extraopts=_extra_opts)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'VAR = "1"',
                                 },
                             ],
                             )
    def test_rulefile_and_subrules(self, input_):
        __cnt = '''
        {
            "oelint.var.suggestedvar": "info",
            "oelint.var.suggestedvar.AUTHOR": "error",
            "oelint.var.suggestedvar.BBCLASSEXTEND": "error",
            "oelint.var.suggestedvar.BUGTRACKER": "error",
            "oelint.var.suggestedvar.SECTION": "error",
            "oelint.var.suggestedvar.CVE_PRODUCT": "info"
        }
        '''
        _extra_opts = [
            '--rulefile={file}'.format(file=self._create_tempfile('rulefile', __cnt)), '--hide', 'info']
        self.check_for_id(self._create_args(input_, _extra_opts),
                          'oelint.var.suggestedvar.CVE_PRODUCT', 0)

    def test_rulefile_subrules_printout(self, capsys):
        from oelint_adv.__main__ import print_rulefile
        __cnt = {
            "oelint.var.suggestedvar.BUGTRACKER": "error",
        }
        _extra_opts = ['--rulefile={file}'.format(file=self._create_tempfile('rulefile', json.dumps(__cnt)))]

        print_rulefile(self._create_args_plain(['--print-rulefile'], _extra_opts))
        captured = capsys.readouterr()
        assert json.loads(captured.out) == __cnt

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'VAR = "1"',
                                 },
                             ],
                             )
    def test_rulefile_default_severity(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _rule_file = self._create_tempfile(
            'rulefile', '{"oelint.var.mandatoryvar": ""}')
        _args = self._create_args(
            input_, extraopts=[f'--rulefile={_rule_file}'])
        for issue in [x[1] for x in run(_args)]:
            assert ':error:' in issue

    @pytest.mark.parametrize('id_', ['oelint.var.override'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'optee-client_3.11.0.bb':
                                     '''
                                     require optee-client.inc
                                     SRCREV = "c0c925384c1d7e3558d27d2708857482952d7907"
                                     ''',
                                     'optee-client.inc':
                                     '''
                                     SUMMARY = "OP-TEE Client API"
                                     DESCRIPTION = "Open Portable Trusted Execution Environment - Normal World Client side of the TEE"
                                     HOMEPAGE = "https://www.op-tee.org/"
                                     LICENSE = "BSD-2-Clause"
                                     LIC_FILES_CHKSUM = "file://LICENSE;md5=69663ab153298557a59c67a60a743e5b"
                                     inherit systemd update-rc.d cmake
                                     SRC_URI = " \\
                                         git://github.com/OP-TEE/optee_client.git \\
                                         file://tee-supplicant.service \\
                                         file://tee-supplicant.sh \\
                                     "
                                     S = "${WORKDIR}/git"
                                     EXTRA_OECMAKE = " \\
                                         -DBUILD_SHARED_LIBS=ON \\
                                         -DCFG_TEE_FS_PARENT_PATH='${localstatedir}/lib/tee' \\
                                     "
                                     EXTRA_OECMAKE_append_toolchain-clang = " -DCFG_WERROR=0"
                                     do_install_append() {
                                         install -D -p -m0644 ${WORKDIR}/tee-supplicant.service ${D}${systemd_system_unitdir}/tee-supplicant.service
                                         install -D -p -m0755 ${WORKDIR}/tee-supplicant.sh ${D}${sysconfdir}/init.d/tee-supplicant
                                         sed -i -e s:@sysconfdir@:${sysconfdir}:g \\
                                             -e s:@sbindir@:${sbindir}:g \\
                                                 ${D}${systemd_system_unitdir}/tee-supplicant.service \\
                                                 ${D}${sysconfdir}/init.d/tee-supplicant
                                     }
                                     SYSTEMD_SERVICE_${PN} = "tee-supplicant.service"
                                     INITSCRIPT_PACKAGES = "${PN}"
                                     INITSCRIPT_NAME_${PN} = "tee-supplicant"
                                     INITSCRIPT_PARAMS_${PN} = "start 10 1 2 3 4 5 . stop 90 0 6 ."
                                     ''',
                                     'optee-examples_3.11.0.bb':
                                     '''
                                     require optee-examples.inc
                                     SRCREV = "9a7dc598591990349d88b4dba3a37aadd6851295"
                                     ''',
                                     'optee-examples_%.bbappend':
                                     'A = "1"',
                                     'optee-examples.inc':
                                     '''
                                     SUMMARY = "OP-TEE examples"
                                     DESCRIPTION = "Open Portable Trusted Execution Environment - Sample Applications"
                                     HOMEPAGE = "https://github.com/linaro-swg/optee_examples"
                                     LICENSE = "BSD-2-Clause"
                                     LIC_FILES_CHKSUM = "file://LICENSE;md5=cd95ab417e23b94f381dafc453d70c30"
                                     DEPENDS = "optee-client optee-os python3-pycryptodome-native"
                                     inherit python3native
                                     require optee.inc
                                     SRC_URI = "git://github.com/linaro-swg/optee_examples.git \\
                                             file://0001-make-Pass-ldflags-during-link.patch \\
                                             "
                                     EXTRA_OEMAKE += "TA_DEV_KIT_DIR=${TA_DEV_KIT_DIR} \\
                                                     HOST_CROSS_COMPILE=${HOST_PREFIX} \\
                                                     TA_CROSS_COMPILE=${HOST_PREFIX} \\
                                                     OUTPUT_DIR=${B} \\
                                                 "
                                     S = "${WORKDIR}/git"
                                     B = "${WORKDIR}/build"
                                     do_compile() {
                                         oe_runmake -C ${S}
                                     }
                                     do_compile[cleandirs] = "${B}"
                                     do_install () {
                                         mkdir -p ${D}${nonarch_base_libdir}/optee_armtz
                                         mkdir -p ${D}${bindir}
                                         install -D -p -m0755 ${B}/ca/* ${D}${bindir}
                                         install -D -p -m0444 ${B}/ta/* ${D}${nonarch_base_libdir}/optee_armtz
                                     }
                                     FILES_${PN} += "${nonarch_base_libdir}/optee_armtz/"
                                     # Imports machine specific configs from staging to build
                                     PACKAGE_ARCH = "${MACHINE_ARCH}"
                                     ''',
                                     'optee.inc':
                                     '''
                                     COMPATIBLE_MACHINE ?= "invalid"
                                     COMPATIBLE_MACHINE_qemuarm64 ?= "qemuarm64"
                                     # Please add supported machines below or set it in .bbappend or .conf
                                     OPTEEMACHINE ?= "${MACHINE}"
                                     OPTEEMACHINE_aarch64_qemuall ?= "vexpress-qemu_armv8a"
                                     OPTEE_ARCH = "null"
                                     OPTEE_ARCH_armv7a = "arm32"
                                     OPTEE_ARCH_aarch64 = "arm64"
                                     OPTEE_CORE = "${@d.getVar('OPTEE_ARCH').upper()}"
                                     OPTEE_TOOLCHAIN = "${@d.getVar('TOOLCHAIN') or 'gcc'}"
                                     OPTEE_COMPILER = "${@bb.utils.contains("BBFILE_COLLECTIONS", "clang-layer", "${OPTEE_TOOLCHAIN}", "gcc", d)}"
                                     # Set here but not passed to EXTRA_OEMAKE by default as that breaks
                                     # the optee-os build
                                     TA_DEV_KIT_DIR = "${STAGING_INCDIR}/optee/export-user_ta"
                                     EXTRA_OEMAKE += "V=1 \\
                                                     LIBGCC_LOCATE_CFLAGS=--sysroot=${STAGING_DIR_HOST} \\
                                                     COMPILER=${OPTEE_COMPILER} \\
                                                     OPTEE_CLIENT_EXPORT=${STAGING_DIR_HOST}${prefix} \\
                                                     TEEC_EXPORT=${STAGING_DIR_HOST}${prefix} \\
                                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_grouping(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.override'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'optee-client_3.11.0.bb':
                                     '''
                                     require optee-client.inc
                                     SRCREV = "c0c925384c1d7e3558d27d2708857482952d7907"
                                     ''',
                                     'optee-client.inc':
                                     '''
                                     SUMMARY = "OP-TEE Client API"
                                     DESCRIPTION = "Open Portable Trusted Execution Environment - Normal World Client side of the TEE"
                                     HOMEPAGE = "https://www.op-tee.org/"
                                     LICENSE = "BSD-2-Clause"
                                     LIC_FILES_CHKSUM = "file://LICENSE;md5=69663ab153298557a59c67a60a743e5b"
                                     inherit systemd update-rc.d cmake
                                     SRC_URI = " \\
                                         git://github.com/OP-TEE/optee_client.git \\
                                         file://tee-supplicant.service \\
                                         file://tee-supplicant.sh \\
                                     "
                                     S = "${WORKDIR}/git"
                                     EXTRA_OECMAKE = " \\
                                         -DBUILD_SHARED_LIBS=ON \\
                                         -DCFG_TEE_FS_PARENT_PATH='${localstatedir}/lib/tee' \\
                                     "
                                     EXTRA_OECMAKE_append_toolchain-clang = " -DCFG_WERROR=0"
                                     do_install_append() {
                                         install -D -p -m0644 ${WORKDIR}/tee-supplicant.service ${D}${systemd_system_unitdir}/tee-supplicant.service
                                         install -D -p -m0755 ${WORKDIR}/tee-supplicant.sh ${D}${sysconfdir}/init.d/tee-supplicant
                                         sed -i -e s:@sysconfdir@:${sysconfdir}:g \\
                                             -e s:@sbindir@:${sbindir}:g \\
                                                 ${D}${systemd_system_unitdir}/tee-supplicant.service \\
                                                 ${D}${sysconfdir}/init.d/tee-supplicant
                                     }
                                     SYSTEMD_SERVICE_${PN} = "tee-supplicant.service"
                                     INITSCRIPT_PACKAGES = "${PN}"
                                     INITSCRIPT_NAME_${PN} = "tee-supplicant"
                                     INITSCRIPT_PARAMS_${PN} = "start 10 1 2 3 4 5 . stop 90 0 6 ."
                                     ''',
                                     'optee-examples_3.11.0.bb':
                                     '''
                                     require optee-examples.inc
                                     SRCREV = "9a7dc598591990349d88b4dba3a37aadd6851295"
                                     ''',
                                     'optee-examples_%.bbappend':
                                     'A = "1"',
                                     'optee-examples.inc':
                                     '''
                                     SUMMARY = "OP-TEE examples"
                                     DESCRIPTION = "Open Portable Trusted Execution Environment - Sample Applications"
                                     HOMEPAGE = "https://github.com/linaro-swg/optee_examples"
                                     LICENSE = "BSD-2-Clause"
                                     LIC_FILES_CHKSUM = "file://LICENSE;md5=cd95ab417e23b94f381dafc453d70c30"
                                     DEPENDS = "optee-client optee-os python3-pycryptodome-native"
                                     inherit python3native
                                     require optee.inc
                                     SRC_URI = "git://github.com/linaro-swg/optee_examples.git \\
                                             file://0001-make-Pass-ldflags-during-link.patch \\
                                             "
                                     EXTRA_OEMAKE += "TA_DEV_KIT_DIR=${TA_DEV_KIT_DIR} \\
                                                     HOST_CROSS_COMPILE=${HOST_PREFIX} \\
                                                     TA_CROSS_COMPILE=${HOST_PREFIX} \\
                                                     OUTPUT_DIR=${B} \\
                                                 "
                                     S = "${WORKDIR}/git"
                                     B = "${WORKDIR}/build"
                                     do_compile() {
                                         oe_runmake -C ${S}
                                     }
                                     do_compile[cleandirs] = "${B}"
                                     do_install () {
                                         mkdir -p ${D}${nonarch_base_libdir}/optee_armtz
                                         mkdir -p ${D}${bindir}
                                         install -D -p -m0755 ${B}/ca/* ${D}${bindir}
                                         install -D -p -m0444 ${B}/ta/* ${D}${nonarch_base_libdir}/optee_armtz
                                     }
                                     FILES_${PN} += "${nonarch_base_libdir}/optee_armtz/"
                                     # Imports machine specific configs from staging to build
                                     PACKAGE_ARCH = "${MACHINE_ARCH}"
                                     ''',
                                     'optee.inc':
                                     '''
                                     COMPATIBLE_MACHINE ?= "invalid"
                                     COMPATIBLE_MACHINE_qemuarm64 ?= "qemuarm64"
                                     # Please add supported machines below or set it in .bbappend or .conf
                                     OPTEEMACHINE ?= "${MACHINE}"
                                     OPTEEMACHINE_aarch64_qemuall ?= "vexpress-qemu_armv8a"
                                     OPTEE_ARCH = "null"
                                     OPTEE_ARCH_armv7a = "arm32"
                                     OPTEE_ARCH_aarch64 = "arm64"
                                     OPTEE_CORE = "${@d.getVar('OPTEE_ARCH').upper()}"
                                     OPTEE_TOOLCHAIN = "${@d.getVar('TOOLCHAIN') or 'gcc'}"
                                     OPTEE_COMPILER = "${@bb.utils.contains("BBFILE_COLLECTIONS", "clang-layer", "${OPTEE_TOOLCHAIN}", "gcc", d)}"
                                     # Set here but not passed to EXTRA_OEMAKE by default as that breaks
                                     # the optee-os build
                                     TA_DEV_KIT_DIR = "${STAGING_INCDIR}/optee/export-user_ta"
                                     EXTRA_OEMAKE += "V=1 \\
                                                     LIBGCC_LOCATE_CFLAGS=--sysroot=${STAGING_DIR_HOST} \\
                                                     COMPILER=${OPTEE_COMPILER} \\
                                                     OPTEE_CLIENT_EXPORT=${STAGING_DIR_HOST}${prefix} \\
                                                     TEEC_EXPORT=${STAGING_DIR_HOST}${prefix} \\
                                                     "
                                     ''',
                                 }
                             ],
                             )
    def test_grouping_with_missing_files(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, extraopts=[
                          '/does/not/exist']), id, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.multiinclude'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'u-boot-rahix.bb':
                                     '''
                                     require u-boot-rahix-common.inc
                                     ''',
                                     'u-boot-rahix-tools.bb':
                                     '''
                                     require u-boot-rahix-common.inc
                                     ''',
                                     'u-boot-rahix-common.inc':
                                     '''
                                     require recipes-bsp/u-boot/u-boot-common.inc
                                     ''',
                                 }
                             ],
                             )
    def test_grouping_with_noversion(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.multiinclude'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'test.bb':
                                     '''
                                     require test.inc
                                     ''',
                                     'test_1.2.3.bb':
                                     '''
                                     require test.inc
                                     ''',
                                     'test_git.bb':
                                     '''
                                     require test.inc
                                     ''',
                                     'test_1.%.bbappend':
                                     '''
                                     require test2.inc
                                     ''',
                                     'test_git.bbappend':
                                     '''
                                     require test2.inc
                                     ''',
                                     'test.inc':
                                     '''
                                     A = "1"
                                     ''',
                                     'test2.inc':
                                     '''
                                     B = "1"
                                     ''',
                                 }
                             ],
                             )
    def test_grouping_with_noversion2(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_hide_info(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(input_, extraopts=['--hide', 'info'])
        issues = [x[1] for x in run(_args)]
        assert (not any([x for x in issues if ':info:' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_hide_warning(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(input_, extraopts=['--hide', 'warning'])
        issues = [x[1] for x in run(_args)]
        assert (not any([x for x in issues if ':warning:' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     FILES = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_hide_error(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(input_, extraopts=['--hide', 'error'])
        issues = [x[1] for x in run(_args)]
        assert (not any([x for x in issues if ':error:' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_noinfo(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(input_, extraopts=['--noinfo'])
        issues = [x[1] for x in run(_args)]
        assert (not any([x for x in issues if ':info:' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_nowarn(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(input_, extraopts=['--nowarn'])
        issues = [x[1] for x in run(_args)]
        assert (not any([x for x in issues if ':warning:' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 }
                             ],
                             )
    def test_messageformat_1(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(
            input_, extraopts=['--messageformat="BAR:FOO"'])
        issues = [x[1] for x in run(_args)]
        assert (any([x for x in issues if 'BAR:FOO' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP:${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_messageformat_2(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(
            input_, extraopts=['--messageformat="{id}:{severity}:{msg}"'])
        issues = [x[1] for x in run(_args)]
        assert (any([x for x in issues if 'oelint.vars.insaneskip:error:' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP:${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_messageformat_wikiurl(self, input_):
        # local imports only
        from oelint_adv.__main__ import run
        from oelint_adv.version import __version__

        _args = self._create_args(
            input_, extraopts=['--messageformat="{wikiurl}"'])
        issues = [x[1] for x in run(_args)]
        assert (any(
            [x for x in issues if f'https://github.com/priv-kweihmann/oelint-adv/blob/{__version__}/docs/wiki/oelint.vars.insaneskip.md' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_messageformat_2_old(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(
            input_, extraopts=['--messageformat="{id}:{severity}:{msg}"', '--release=dunfell'])
        issues = [x[1] for x in run(_args)]
        assert (any([x for x in issues if 'oelint.vars.insaneskip:error:' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_rulefile(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _cstfile = self._create_tempfile('constants.json', '{}')

        _args = self._create_args(
            input_, extraopts=['--rulefile={file}'.format(file=_cstfile)])
        issues = [x[1] for x in run(_args)]
        assert (any(issues))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                    A = "1"
                                    ''',
                                 },
                             ],
                             )
    def test_rulefile_subids_only(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _rule_file = {
            "oelint.var.mandatoryvar.DESCRIPTION": "error",
            "oelint.var.mandatoryvar.LICENSE": "error",
            "oelint.var.mandatoryvar.SRC_URI": "error"
        }

        _cstfile = self._create_tempfile('rules.json', json.dumps(_rule_file))

        _args = self._create_args(
            input_, extraopts=['--rulefile={file}'.format(file=_cstfile)])
        issues = [x[1] for x in run(_args)]
        assert (not any(any(x.startswith(y) for y in _rule_file.keys()) for x in issues))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'DESCRIPTION = "foo"',
                                 },
                             ],
                             )
    def test_rulefile_filtering(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _cstfile = self._create_tempfile(
            'constants.json', '{"oelint.var.mandatoryvar.DESCRIPTION": "warning", "oelint.var.mandatoryvar": "info" }')

        _args = self._create_args(
            input_, extraopts=['--rulefile={file}'.format(file=_cstfile), '--hide', 'info'])
        issues = [x[1] for x in run(_args)]
        assert (not any(issues))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'DESCRIPTION = "foo"',
                                 },
                             ],
                             )
    def test_rulefile_filtering2(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _cstfile = self._create_tempfile(
            'constants.json', '{"oelint.var.mandatoryvar.DESCRIPTION": "warning"}')

        _args = self._create_args(
            input_, extraopts=['--rulefile={file}'.format(file=_cstfile)])
        issues = [x[1] for x in run(_args)]
        assert (not any("oelint.var.mandatoryvar.DESCRIPTION" in x for x in issues))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     'HOMEPAGE = "foo"',
                                 },
                             ],
                             )
    def test_rulefile_filtering_invert(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _cstfile = self._create_tempfile(
            'constants.json', '{"oelint.var.mandatoryvar.DESCRIPTION": "warning", "oelint.var.mandatoryvar": "info" }')

        _args = self._create_args(
            input_, extraopts=['--rulefile={file}'.format(file=_cstfile), '--hide', 'info'])
        issues = [x[1] for x in run(_args)]
        assert (any(issues))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_broken_rulefile(self, input_):
        with pytest.raises(ArgumentTypeError):
            _args = self._create_args(
                input_, extraopts=['--rulefile=/does/not/exist'])

    @pytest.mark.parametrize('input_',
                             [
                                 {'oelint.var.mandatoryvar.DESCRIPTION': 'warning'},
                                 {'oelint.var.mandatoryvar': 'warning'},
                                 {'oelint.var.mandatoryvar': 'warning',
                                  'oelint.var.mandatoryvar.DESCRIPTION': 'info'},
                             ],
                             )
    def test_print_rulefile(self, capsys, input_):
        # local imports only
        from oelint_adv.__main__ import print_rulefile

        _rule_file = self._create_tempfile('rules.json', json.dumps(input_))
        _args = self._create_args(
            {}, extraopts=[f'--rulefile={_rule_file}', '--print-rulefile'])
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)
        for k, v in input_.items():
            assert out[k] == v

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_nonquiet(self, input_):
        # local imports only
        from oelint_adv.__main__ import arguments_post
        from oelint_adv.__main__ import run

        _args = arguments_post(self._create_args_parser().parse_args(
            [self._create_tempfile(k, v) for k, v in input_.items()]
        ))
        issues = [x[1] for x in run(_args)]
        assert (any(issues))

    def test_invalidfile(self):
        # local imports only
        from oelint_adv.__main__ import arguments_post
        from oelint_adv.__main__ import run

        _args = arguments_post(self._create_args_parser().parse_args(
            ['/does/not/exist']
        ))
        issues = [x[1] for x in run(_args)]
        assert (not any(issues))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_exit_zero(self, input_):
        _args = self._create_args(input_, extraopts=['--exit-zero'])
        assert (_args.exit_zero)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test_2.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint adv-test_1.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_sorted_by_file_and_line(self, capsys, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(input_)
        issues = [x[0] for x in run(_args)]

        assert sorted(issues, key=lambda x: x[0]) == issues

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test_2.bb':
                                     '''
                                     VAR = "1"
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 }
                             ],
                             )
    def test_suppress(self, capsys, input_):
        _args = self._create_args(input_, ['--suppress="a b c"', '--suppress="d"'])

        assert "a" in _args.suppress
        assert "b" in _args.suppress
        assert "c" in _args.suppress
        assert "d" in _args.suppress

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     ''',
                                 }
                             ],
                             )
    def test_release_author(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(
            input_, extraopts=['--release=sumo'])
        issues = [x[1] for x in run(_args)]
        assert (any([x for x in issues if 'AUTHOR' in x]))

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     ''',
                                 }
                             ],
                             )
    def test_release_no_author(self, input_):
        # local imports only
        from oelint_adv.__main__ import run

        _args = self._create_args(
            input_, extraopts=['--release=nanbield'])
        issues = [x[1] for x in run(_args)]
        assert not any([x for x in issues if 'AUTHOR' in x])

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     VAR = "1"
                                     ''',
                                 }
                             ],
                             )
    def test_release_invalid(self, input_):
        with pytest.raises(SystemExit):
            self._create_args(input_, extraopts=['--release=doesnotexist'])



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsBBVars(TestBaseClass):

    def __generate_sample_code(self, var, operation):
        return '''
            {var} {operation} "foo"
            '''.format(var=var, operation=operation)

    @pytest.mark.parametrize('id_', ['oelint.vars.bbvars'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        'BB_CONSOLELOG',
        'BB_CURRENTTASK',
        'BB_DANGLINGAPPENDS_WARNONLY',
        'BB_DEFAULT_TASK',
        'BB_DISKMON_DIRS',
        'BB_DISKMON_WARNINTERVAL',
        'BB_ENV_EXTRAWHITE',
        'BB_ENV_WHITELIST',
        'BB_FETCH_PREMIRRORONLY',
        'BB_FILENAME',
        'BB_GENERATE_MIRROR_TARBALLS',
        'BB_HASHBASE_WHITELIST',
        'BB_HASHCHECK_FUNCTION',
        'BB_HASHCONFIG_WHITELIST',
        'BB_INVALIDCONF',
        'BB_LOGFMT',
        'BB_NICE_LEVEL',
        'BB_NO_NETWORK',
        'BB_NUMBER_PARSE_THREADS',
        'BB_NUMBER_THREADS',
        'BB_ORIGENV',
        'BB_PRESERVE_ENV',
        'BB_RUNFMT',
        'BB_RUNTASK',
        'BB_SCHEDULER',
        'BB_SCHEDULERS',
        'BB_SETSCENE_DEPVALID',
        'BB_SETSCENE_VERIFY_FUNCTION',
        'BB_SIGNATURE_EXCLUDE_FLAGS',
        'BB_SIGNATURE_HANDLER',
        'BB_SRCREV_POLICY',
        'BB_STAMP_POLICY',
        'BB_STAMP_WHITELIST',
        'BB_STRICT_CHECKSUM',
        'BB_TASK_NICE_LEVEL',
        'BB_TASKHASH',
        'BB_VERBOSE_LOGS',
        'BB_WORKERCONTEXT',
        'BBDEBUG',
        'BBFILE_COLLECTIONS',
        'BBFILE_PATTERN',
        'BBFILE_PRIORITY',
        'BBFILES',
        'BBINCLUDED',
        'BBINCLUDELOGS',
        'BBINCLUDELOGS_LINES',
        'BBLAYERS',
        'BBMASK',
        'BBPATH',
        'BBSERVER',
        'BBVERSIONS',
        'BITBAKE_UI',
        'BUILDNAME',
        'CACHE',
        'DL_DIR',
        'FILE',
        'FILESDIR',
        'FILESPATH',
        'INHERIT',
        'LAYERDEPENDS',
        'LAYERDIR',
        'LAYERVERSION',
        'MIRRORS',
        'MULTI_PROVIDER_WHITELIST',
        'PERSISTENT_DIR',
        'PREFERRED_PROVIDER',
        'PREFERRED_PROVIDERS',
        'PREFERRED_VERSION',
        'PREMIRRORS',
        'PRSERV_HOST',
        'STAMP',
        'TOPDIR',
    ])
    @pytest.mark.parametrize('operation', ['=', ':=', '.=', '=.', '+=', '=+', ' =+'])
    def test_bad(self, id_, occurrence, var, operation):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(var, operation),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.bbvars'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        'BB_CONSOLELOG',
        'BB_CURRENTTASK',
        'BB_DANGLINGAPPENDS_WARNONLY',
        'BB_DEFAULT_TASK',
        'BB_DISKMON_DIRS',
        'BB_DISKMON_WARNINTERVAL',
        'BB_ENV_EXTRAWHITE',
        'BB_ENV_WHITELIST',
        'BB_FETCH_PREMIRRORONLY',
        'BB_FILENAME',
        'BB_GENERATE_MIRROR_TARBALLS',
        'BB_HASHBASE_WHITELIST',
        'BB_HASHCHECK_FUNCTION',
        'BB_HASHCONFIG_WHITELIST',
        'BB_INVALIDCONF',
        'BB_LOGFMT',
        'BB_NICE_LEVEL',
        'BB_NO_NETWORK',
        'BB_NUMBER_PARSE_THREADS',
        'BB_NUMBER_THREADS',
        'BB_ORIGENV',
        'BB_PRESERVE_ENV',
        'BB_RUNFMT',
        'BB_RUNTASK',
        'BB_SCHEDULER',
        'BB_SCHEDULERS',
        'BB_SETSCENE_DEPVALID',
        'BB_SETSCENE_VERIFY_FUNCTION',
        'BB_SIGNATURE_EXCLUDE_FLAGS',
        'BB_SIGNATURE_HANDLER',
        'BB_SRCREV_POLICY',
        'BB_STAMP_POLICY',
        'BB_STAMP_WHITELIST',
        'BB_STRICT_CHECKSUM',
        'BB_TASK_NICE_LEVEL',
        'BB_TASKHASH',
        'BB_VERBOSE_LOGS',
        'BB_WORKERCONTEXT',
        'BBDEBUG',
        'BBFILE_COLLECTIONS',
        'BBFILE_PATTERN',
        'BBFILE_PRIORITY',
        'BBFILES',
        'BBINCLUDED',
        'BBINCLUDELOGS',
        'BBINCLUDELOGS_LINES',
        'BBLAYERS',
        'BBMASK',
        'BBPATH',
        'BBSERVER',
        'BBVERSIONS',
        'BITBAKE_UI',
        'BUILDNAME',
        'CACHE',
        'DEFAULT_PREFERENCE',
        'DL_DIR',
        'FILE',
        'FILESDIR',
        'FILESPATH',
        'INHERIT',
        'LAYERDEPENDS',
        'LAYERDIR',
        'LAYERVERSION',
        'MIRRORS',
        'MULTI_PROVIDER_WHITELIST',
        'PERSISTENT_DIR',
        'PREFERRED_PROVIDER',
        'PREFERRED_PROVIDERS',
        'PREFERRED_VERSION',
        'PREMIRRORS',
        'PRSERV_HOST',
        'STAMP',
        'TOPDIR',
    ])
    @pytest.mark.parametrize('operation', ['?=', '??=', ' ??='])
    def test_good_weak(self, id_, occurrence, var, operation):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(var, operation),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)
